/*

	An Adobe Illustrator L-System through Scriptographer v1.1
	by Hakan Lundgren
	 2011 Monovektor.com
	
	
	A quick introduction of L-Systems:
	http://en.wikipedia.org/wiki/Lsystem
	
	
	
	
	
	The L-System object takes two arguments.
		- angle: a number by how much a full circle is divided (ie. angle: 4 = 90).
		- axiom: the starting value (a string) of the system.
	
	
	
	The rules have two or three arguments depending on wether the system is stochastic or not.
		- rule: a single letter/char string.
		- grammar: a string (or an array of strings IF probabilities are used) that replace the rule char.
		- probabilities: an array of numbers indicating the probability of its corresponding grammar. For instance,
		  if grammar is an array of two or more strings, probabilities would then be an array of equal length such:

			  rule: 'F'
			  grammar: ['F[+F]-F', 'F[+F+F]-']
			  probabilities: [0.2, 0.8]
			  
		  Would look like this:

			  	new Rule('F', ['F[+F]-F', 'F[+F+F]-'], [0.2, 0.8])
	
		  Whenever an F occurs it will be substituted by either one of the grammars according to each grammars probability.
		  The first one will only be picked about 20% of the times. It is important that the numbers in the probabilities
		  array adds up to exactly 1 (one). A L-System with this particular feature is called a stochastic L-system.
		- If however no probabilities are used (as in most cases) only one rule and grammar string are needed.
		- A grammar cannot have an empty string (''), if no substitution is to be made use a space (' ') instead.
	
	
	
	The draw function also takes two or three arguments.
	
		lsystem.draw(iter, lineLength, lineModification)
	
		- iter: number of times to iterate through the initial axiom. The alphabet grows exponentially so beware of high
		  numbers, usually a value between 4 - 9 is sufficient. Sometimes as low as 2 - 6.
		- lineLength: the initial length of a line meassured in points. Most grammars never change the length at all.
		- lineModification: a multiplier (and divider) by how much a line will grow och shrink. ~0.9-1.1 seem to yield the best
		  results as any lower/higher value will radically change the length.
	
	
	
	A typical L-System will look like this:
		var lsystem = new Lsystem(8, '++FX')
		lsystem.rules.push(new Rule('X', '<<<<[-FX]+FX'))
			::
		lsystem.rules.push(...More rules, if any)
			::
		lsystem.draw(12, 40, 0.9)
	
	
	
	The constants used are:
		F	Moves forward one step (length defined by the lineLength variable) and draws a line.
		f	Moves forward one step without drawing a line.
		+	Changes the angle in a counter clockwise direction.
		-	Changes the angle in a clockwise direction.
		|	Turns 180
		<	Multiplies the current line length by a number defined by the lineModification variable
		>	Divide the current line length by a number defined by the lineModification variable
		[	Create a branch, similar to the array method push()
		]	End branch and return to its parrent, similar to the array method pop()
		
	More constants may be added later.
	
	
	
	At the bottom is a collection of nice looking grammars I have found all over the place.
	More reading about L-Systems (written by the inventor of it all) can be found in the book The Algorithmic Beauty of
	Plants available for free as a high quality PDF here:
	http://algorithmicbotany.org/papers/abop/abop.pdf
	
*/


var Lsystem = function(angle, axiom){
			this.angle = angle
			this.axiom = axiom
			this.rules = []
}


Lsystem.prototype.generateAlphabet = function(iterations){
			this.alphabet = this.axiom

			var temp, W
			// Iterator loop
			for(var i = 0; i < iterations; i++){
					temp = ''

					// Loop through all letters
					for(var j = 0; j < this.alphabet.length; j++){

							W = this.alphabet.charAt(j)

							// Find rule
							for(var k in this.rules){
									if(this.rules[k].name == W){
											W = this.rules[k].getRule()
									}
							}

							temp += W
					}

					this.alphabet = temp
			}

			return this.alphabet
}


Lsystem.prototype.draw = function(iter, lineLength, lineModification){
	
			this.generateAlphabet(iter)

			var vector = new Point({ length: lineLength, angle: 0 })
			var branches = []
			var path = new Path()
			var currentPosition = new Point(0, 0)
			path.add(currentPosition)

			for(var i = 0; i < this.alphabet.length; i++){
				
					switch(this.alphabet.charAt(i)){
							case('F'):												// Move forward and draw line
									currentPosition += vector
									path.lineTo(currentPosition)
									break
							case('f'):												// Move forward without drawing a line
									currentPosition += vector
									path = new Path()
									path.add(currentPosition)
									break
							case('+'):												// Change angle counter clockwise
									vector.angle -= 360 / this.angle
									break
							case('-'):												// Change angle clockwise
									vector.angle += 360 / this.angle
									break
							case('|'):												// Turn around 180
									vector.angle += 180
									break
							case('<'):												// Increases length of line if lineModification > 1, otherwise decrease
									vector.length *= lineModification
									break
							case('>'):												// Decreases length of line if lineModification > 1, otherwise increase
									vector.length /= lineModification
									break
							case('['):												// Adds another branch
									branches.push([path, currentPosition, vector])
									vector = new Point(vector)
									path = new Path()
									path.add(currentPosition)
									break
							case(']'):												// Returns to the parrent branch
									branchArray = branches[branches.length -1]
									path = branchArray[0]
									currentPosition = branchArray[1]
									vector = branchArray[2]
									branches.pop()
									break				
					}
			}
}


var Rule = function(rule, grammar, probabilities){
			this.name = rule
			this.grammar = grammar

			// Sets up a probability array
			if(probabilities && probabilities.length > 1){
					var tempArray = []
					var previousValue = nextValue = 0

					for(var i = 0; i < probabilities.length; i++){
							nextValue = previousValue + probabilities[i]
							tempArray.push(nextValue)
							previousValue = nextValue
					}

					if(previousValue == 1){ this.probability = tempArray }
			}
}


Rule.prototype.getRule = function(){
			if(this.probability){
					var randomRule = Math.random()
					var counter = 0

					while(randomRule > this.probability[counter]){
							counter++
					}

					return this.grammar[counter]
			}

			return this.grammar
}


/*	


	lsystem.draw(iter, lineLength, lineModification)
		iter = Number of iterations
		lineLength = Length of each line in points
		lineModification = Amount by which to multiply/divide the line length (preferably ~ 0.9 to 1.1)
*/
var lsystem = new Lsystem(4, 'F')
lsystem.rules.push(new Rule('F', 'F-F+F'))
lsystem.draw(8, 10, 0.95)

/*

These are a bunch of grammars I've found on the internet. Try them out, they're nice looking.
			
		
Misc.
				
		Zig-Zag
				4, 'F'
				new Rule('F', 'F-F+F')
*/